/*
 * Copyright (c) 2007, 2008, 2009, 2010, 2011 David Berkman
 * 
 * This file is part of the SmallMind Code Project.
 * 
 * The SmallMind Code Project is free software, you can redistribute
 * it and/or modify it under the terms of GNU Affero General Public
 * License as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 * 
 * The SmallMind Code Project is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the the GNU Affero General Public
 * License, along with The SmallMind Code Project. If not, see
 * <http://www.gnu.org/licenses/>.
 * 
 * Additional permission under the GNU Affero GPL version 3 section 7
 * ------------------------------------------------------------------
 * If you modify this Program, or any covered work, by linking or
 * combining it with other code, such other code is not for that reason
 * alone subject to any of the requirements of the GNU Affero GPL
 * version 3.
 */
package org.smallmind.persistence.cache.aop;

import java.util.Iterator;
import org.smallmind.persistence.Durable;
import org.smallmind.persistence.PersistenceMode;
import org.smallmind.persistence.cache.VectoredDao;

public class CacheCoherentIterator<I extends Comparable<I>, D extends Durable<I>> implements Iterator<D>, Iterable<D> {

  private Iterator<D> durableIter;
  private VectoredDao<I, D> vectoredDao;
  private Class<D> durableClass;

  public CacheCoherentIterator (Iterator<D> durableIter, Class<D> durableClass, VectoredDao<I, D> vectoredDao) {

    this.durableIter = durableIter;
    this.durableClass = durableClass;
    this.vectoredDao = vectoredDao;
  }

  public boolean hasNext () {

    return durableIter.hasNext();
  }

  public Iterator<D> iterator () {

    return this;
  }

  public D next () {

    return vectoredDao.persist(durableClass, durableIter.next(), PersistenceMode.SOFT);
  }

  public void remove () {

    durableIter.remove();
  }
}
